use pyo3::prelude::*;
use rand::seq::SliceRandom;
use fernet::Fernet;
use std::thread;
use std::sync::mpsc;




#[pyfunction]
fn generate_password() -> PyResult<String> {
    let (tx, rx) = mpsc::channel();

    // Start 10 threads to generate parts of the password
    for _ in 0..10 {
        let tx = tx.clone();
        thread::spawn(move || {
            let password = generate_part_of_password();
            tx.send(password).unwrap();
        });
    }


    let mut passwords = Vec::new();
    // Start receiving the generated passwords
    for _ in 0..10 {
        passwords.push(rx.recv().unwrap());
    }

    // Join the passwords into a single string and return it
    Ok(passwords.join(""))
}

fn generate_part_of_password() -> String {
    let mut rng = rand::thread_rng();

    let digits: Vec<char> = ('0'..='9').collect();
    let locase_characters: Vec<char> = ('a'..='z').collect();
    let upcase_characters: Vec<char> = ('A'..='Z').collect();
    let symbols: Vec<char> = vec!['@', '#', '$', '%', '=', ':', '?', '.', '/', '|', '~', '>', '*', '(', ')', '<'];

    let mut password: Vec<char> = Vec::new();

    password.push(*digits.choose(&mut rng).unwrap());
    password.push(*locase_characters.choose(&mut rng).unwrap());
    password.push(*upcase_characters.choose(&mut rng).unwrap());
    password.push(*symbols.choose(&mut rng).unwrap());

    password.shuffle(&mut rng);

    password.into_iter().collect()
}

#[pyfunction]
fn fernet_encrypt(key: String, message: String) -> PyResult<String> {
    let fernet = Fernet::new(&key).expect("Invalid key");
    let encrypted_message = fernet.encrypt(message.as_bytes());
    Ok(encrypted_message)
}

#[pyfunction]
fn fernet_decrypt(key: String, token: String) -> PyResult<String> {
    let fernet = Fernet::new(&key).expect("Invalid key");
    let decrypted_message = fernet.decrypt(token.as_str()).expect("Token cannot be decrypted");
    Ok(String::from_utf8(decrypted_message).expect("Decrypted token is not valid UTF-8"))
}

#[pymodule]
fn rust_module(_py: Python, m: &PyModule) -> PyResult<()> {
    m.add_function(wrap_pyfunction!(generate_password, m)?)?;
    m.add_function(wrap_pyfunction!(fernet_encrypt, m)?)?;
    m.add_function(wrap_pyfunction!(fernet_decrypt, m)?)?;

    Ok(())
}
