from flask import Blueprint,render_template,request,redirect,flash,url_for
from .models import  Account_record
from . import db
import rust_module
from cryptography.fernet import Fernet
from rust_module import fernet_encrypt, fernet_decrypt
from keys import get_key
auth = Blueprint('auth', __name__)



@auth.route('/')
def home():
    return render_template("home.html")

@auth.route('/new_account', methods=['GET', 'POST'])
def new_account():
    key = get_key().decode()
    cipher_suite = Fernet(get_key())

    if request.method == 'POST':
        encrypt_data(request, key)
        return redirect(url_for('auth.pass_storage'))
    generated_password = cipher_suite.encrypt(rust_module.generate_password().encode())
    generated_password = generated_password.decode()

    return render_template("new_account.html",generated_password=generated_password)


@auth.route('/pass_storage', methods=['GET', 'POST'])
def pass_storage():
    key = get_key().decode()
    if request.method == 'POST':
        id_to_delete = Account_record.query.filter_by(id=request.form.get('id')).first().id
        if id_to_delete is not None:
            Account_record.query.filter_by(id=id_to_delete).delete()
            db.session.commit()
            flash('Account deleted successfully!', category='success')
            return redirect(url_for('auth.pass_storage'))
        else:
            flash('Account not found!', category='error')
            return redirect(url_for('auth.pass_storage'))


    storage = decrypt_data(Account_record.query.all(), key)

    return render_template("pass_storage.html",storage=storage)    

@auth.route('/generate_password', methods=['POST'])
def generate_password():
    dict ={"password": rust_module.generate_password()}
    return dict


def encrypt_data(request, key):
    username = fernet_encrypt(key, request.form.get('username'))
    password = fernet_encrypt(key, request.form.get('password'))

    ar = Account_record(username=username, password=password)
    if request.form.get('url') is not None and request.form.get('url') != "":
        url = fernet_encrypt(key, request.form.get('url'))
        ar.url = url
    if request.form.get('notes') is not None and request.form.get('notes') != "":
        notes = fernet_encrypt(key, request.form.get('notes'))
        ar.notes = notes
    if request.form.get('title') is not None and request.form.get('title') != "":
        title = fernet_encrypt(key, request.form.get('title'))
        ar.title = title
    db.session.add(ar)
    db.session.commit()
    flash('Account created successfully!', category='success')

def decrypt_data(storage, key):
    for i in storage:
        i.username = fernet_decrypt(key, i.username)
        i.password = fernet_decrypt(key, i.password)
        if i.url is not None:
            i.url = fernet_decrypt(key, i.url)
        if i.notes is not None:
            i.notes = fernet_decrypt(key, i.notes)
        if i.title is not None:
            i.title = fernet_decrypt(key, i.title)
    return storage